/*
  Stepper_Axlr8_i.cpp - Stepper library with acceleration / deceleration
  for Wiring/Arduino
  - Version 1.0
 */

#include "WProgram.h"
#include "Stepper_Aclr8_i.h"
/*
 *   Constructor for four-pin stepper motor
 *   Sets which wires should control the motor and
 *   primary variables used by motor.
 */

Stepper_Aclr8_i::Stepper_Aclr8_i(int number_of_steps, \
						   int motor_pin_1, int motor_pin_2, \
						   int motor_pin_3, int motor_pin_4)
{
  // Stepper Motor
  // which step the motor is on
  this->step_number = 0;
  // the motor speed, in revolutions per minute
  this->speed = 0;
  // motor direction
  this->direction = 0;
  // time stamp in ms of the last step taken
  this->last_step_time = 0;
  // total number of steps for this motor
  this->number_of_steps = number_of_steps;
  // find the delay nec at 1 rev per min
  this->delay_1revpermin = 60L * 1000L  / number_of_steps;

  // Arduino pins for the motor control connection
  this->motor_pin_1 = motor_pin_1;
  this->motor_pin_2 = motor_pin_2;
  this->motor_pin_3 = motor_pin_3;
  this->motor_pin_4 = motor_pin_4;

  // setup the pins on the microcontroller
  pinMode(this->motor_pin_1, OUTPUT);
  pinMode(this->motor_pin_2, OUTPUT);
  pinMode(this->motor_pin_3, OUTPUT);
  pinMode(this->motor_pin_4, OUTPUT);
}

/*
  Sets the minimum speed in revs per minute
*/
void Stepper_Aclr8_i::setMinSpeed(long minSpeed)
{
	this->motorMinSpeed = minSpeed;
}

/*
  Sets the maximum speed in revs per minute
*/
void Stepper_Aclr8_i::setMaxSpeed(long maxSpeed)
{
	this->motorMaxSpeed = maxSpeed;
}

/*
	Set the acceleration rate
 */
void Stepper_Aclr8_i::setAcc(double Accel)
{
	// Number of steps to accelerate over
	this->motorAccSteps = Accel;
	// Change in speed needed per step
	this->motorAccelVal = (this->motorMaxSpeed -this->motorMinSpeed) / (Accel-1);
}

/*
	Set the interrupt pins
 */
void Stepper_Aclr8_i::setIpins(int firePin, int readPin)
{
	// Pin to raise high and set as an output pin
	pinMode(this->firePin, OUTPUT);
	this->firePin = firePin;
	// Pin to read
	this->readPin = readPin;
}


/*
	Get the address of the 'steps to do' variable
 */
void Stepper_Aclr8_i::setToDoAddress(int* StepsAddress)
	{
		gStepAdd = StepsAddress;
	}

void Stepper_Aclr8_i::step(int steps_to_move)
{
  // set the interrupt 'fire' pin high
  digitalWrite(this->firePin, HIGH);	
 
  // steps taken
  int steps_no= 1;
  // Steps for each motor to complete, and the total
  // how many steps to take for the motor
//  int steps_left = abs(steps_to_move);
  steps_left = abs(steps_to_move);
  
  // how many steps to take
  int total_steps_left = steps_left;

  // determine direction based on whether steps_to_mode is + or -:
  // First motor
  if (steps_to_move > 0) {this->direction = 1;}
  else
  {this->direction = 0;}

  // reset the motor speed
  this->speed = this->motorMinSpeed;

  // this is the loop for motor
  while(total_steps_left > 0){

    // set a new value at the StepsLeft address (ie StepsToDo variable)
    *gStepAdd = total_steps_left;
	  
	// read the interrupt pin
    int readVal = digitalRead(this->readPin);
	// if high then exit
	if (readVal==HIGH){
	break;
	}

    //If the motor steps not completed it's steps then
    if (steps_left > 0) {

	  //1.a. if in the first third of total steps AND
	  if ((steps_left > abs(steps_to_move/3)) && \
		  (steps_left > (abs(steps_to_move/3)*2))) {
			  // 1.b if not yet at max speed
			  if (this->speed < this->motorMaxSpeed){
				  // get the new speed
				  this->speed = ((steps_no -1) * this->motorAccelVal) + this->motorMinSpeed; 
				  // get the new delay
				  step_delay = (this->delay_1revpermin / this->speed); 
              } // i.b.  end if: not yet max speed
	  } //1.a. end if: first third of total steps
			  
	  //2. if in middle third of steps do nothing

	  //3.a. if in the last third of total steps AND
	  if (steps_left <= (abs(steps_to_move)/3)) {
		  //3.b. if within deceleration zone
		  if (steps_left <= this->motorAccSteps) {
			  // get the new speed
			  this->speed = ((steps_left-1) * this->motorAccelVal) + this->motorMinSpeed; 
			  // get the new delay
			  step_delay = (this->delay_1revpermin / this->speed); 
		} //end if: within deceleration zone
	  } //3.a. end if: last third of total steps

      // move only if the appropriate delay has passed:
      if (millis() - this->last_step_time >= this->step_delay) {
        // get the timeStamp of when you stepped:
        this->last_step_time = millis();
          // increment or decrement the step number,
          // depending on direction:
          if (this->direction == 1) {
            this->step_number++;
            if (this->step_number == this->number_of_steps) {
            this->step_number = 0;
            }
          }
          else
		  {
            if (this->step_number == 0) {
            this->step_number = this->number_of_steps;
            }
          this->step_number--;
          }
      // in/decrement the steps left:
      steps_no++;
      steps_left--;
	  total_steps_left--;
	  // step the motor to step number 0, 1, 2, or 3:
      stepMotor(this->step_number % 4);
      }
    } //End: if (steps_left > 0)
  } //End: WHILE statement

  // set the interrupt 'fire' pin low
  digitalWrite(this->firePin, LOW);
} //End: void step()


/*
 * Moves the Motor forward or backwards
 */
void Stepper_Aclr8_i::stepMotor(int thisStep)
{
    switch (thisStep) {
      case 0:    // 1010
      digitalWrite(motor_pin_1, HIGH);
      digitalWrite(motor_pin_2, LOW);
      digitalWrite(motor_pin_3, HIGH);
      digitalWrite(motor_pin_4, LOW);
      break;
      case 1:    // 0110
      digitalWrite(motor_pin_1, LOW);
      digitalWrite(motor_pin_2, HIGH);
      digitalWrite(motor_pin_3, HIGH);
      digitalWrite(motor_pin_4, LOW);
      break;
      case 2:    //0101
      digitalWrite(motor_pin_1, LOW);
      digitalWrite(motor_pin_2, HIGH);
      digitalWrite(motor_pin_3, LOW);
      digitalWrite(motor_pin_4, HIGH);
      break;
      case 3:    //1001
      digitalWrite(motor_pin_1, HIGH);
      digitalWrite(motor_pin_2, LOW);
      digitalWrite(motor_pin_3, LOW);
      digitalWrite(motor_pin_4, HIGH);
      break;
    }
}

/*
  version() returns the version of the library:
*/
int Stepper_Aclr8_i::version(void)
{
  return 1;
}